package com.example.nan_scan;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.os.Build;
import androidx.core.app.ActivityCompat;
import com.journeyapps.barcodescanner.BarcodeCallback;
import com.journeyapps.barcodescanner.BarcodeResult;
import com.journeyapps.barcodescanner.BarcodeView;
import com.journeyapps.barcodescanner.Size;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.PluginRegistry;
import static android.content.pm.PackageManager.PERMISSION_GRANTED;
import static java.lang.Math.min;

public class NanScanViewNew extends BarcodeView implements PluginRegistry.RequestPermissionsResultListener {

    private NanScanCaptureAction scanCaptureAction;
    private Context context;
    private Activity activity;
    private ActivityPluginBinding activityPluginBinding;
    private int CAMERA_REQUEST_CODE = 10086;
    private double vw;
    private double vh;
    private double scale = 1.0;//.7;

    public NanScanViewNew(Context context, Activity activity, ActivityPluginBinding activityPluginBinding, Object args) {
        super(context,null);
        this.context = context;
        this.activity = activity;
        this.activityPluginBinding = activityPluginBinding;
        activityPluginBinding.addRequestPermissionsResultListener(this);
        checkPermission();
    }

    private void checkPermission() {
        if (hasPermission()) {
            start();
        } else {
            String[] permissions = new String[1];
            permissions[0] = Manifest.permission.CAMERA;
            ActivityCompat.requestPermissions(activity, permissions, CAMERA_REQUEST_CODE);
        }
    }

    private void start() {
        this.decodeContinuous(new BarcodeCallback() {
            @Override
            public void barcodeResult(BarcodeResult result) {
                if (scanCaptureAction != null){
                    scanCaptureAction.onScanCapture(result.getText());
                }
            }
        });
        _resume();
    }

    private boolean hasPermission() {
        return Build.VERSION.SDK_INT < Build.VERSION_CODES.M ||
                activity.checkSelfPermission(Manifest.permission.CAMERA) == PERMISSION_GRANTED;
    }

    public void _resume() {
        this.resume();
    }
    public void _pause() {
        this.pause();
    }
    public void toggleTorchMode(boolean mode) {
        this.setTorch(mode);
    }
    public void setScanCaptureAction(NanScanCaptureAction scanCaptureAction) {
        this.scanCaptureAction = scanCaptureAction;
    }

    public void dispose() {
        this.stopDecoding();
        _pause();
    }

    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        super.onLayout(changed, l, t, r, b);
        vw = getWidth();
        vh = getHeight();
        int areaWidth = (int) (min(vw, vh) * scale);
        this.setFramingRectSize(new Size(areaWidth, areaWidth));
    }

    @Override
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        if (requestCode == CAMERA_REQUEST_CODE && grantResults[0] == PERMISSION_GRANTED) {
            start();
            return true;
        }
        return false;
    }


}
