//
//  NanScanView.m
//  nan_scan
//
//  Created by 周泽南 on 2021/8/16.
//

#import "NanScanView.h"
#define WeakSelf __weak typeof(self) weakSelf = self;
//获取屏幕宽度与高度
#define SCREEN_WIDTH   [UIScreen mainScreen].bounds.size.width
#define SCREENH_HEIGHT [UIScreen mainScreen].bounds.size.height

@interface NanScanView()<AVCaptureMetadataOutputObjectsDelegate>
{
    BOOL upOrdown;
    AVCaptureDevice *lightDevice;
    NSObject<FlutterPluginRegistrar>*_registrar;
}
@property (nonatomic,strong) UIView *centerView;//扫描的显示视图
@property (strong,nonatomic) AVCaptureDevice *device;
@property (strong,nonatomic) AVCaptureDeviceInput *input;
@property (strong,nonatomic) AVCaptureMetadataOutput *output;
@property (strong,nonatomic) AVCaptureSession *session;
@property (strong,nonatomic) AVCaptureVideoPreviewLayer *preview;
@property (nonatomic,strong) FlutterMethodChannel *channel;
@property (nonatomic,assign) BOOL isFlashLight;
@end

@implementation NanScanView{
    UIView *_bigView;
}

-(instancetype)initWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args registrar:(NSObject<FlutterPluginRegistrar>*)registrar{
    if (self = [super init]) {
        [self initUI:frame viewIdentifier:viewId registrar:registrar];
    }
    return self;
}

-(void)initUI:(CGRect)frame viewIdentifier:(int64_t)viewId registrar:(NSObject<FlutterPluginRegistrar> *)registrar{
    _bigView = [[UIView alloc]initWithFrame:frame];
    _bigView.backgroundColor = [UIColor whiteColor];
    
    //扫描范围
    _centerView = [[UIView alloc]initWithFrame:CGRectMake(0, 0, CGRectGetWidth(self.view.frame), CGRectGetHeight(self.view.frame))];
    _centerView.backgroundColor = [UIColor clearColor];
    [self.view addSubview:_centerView];
    
    [self setupCamera];

    NSString *viewIdStr = [NSString stringWithFormat:@"nan_scan_method_%lld",viewId];
    _channel = [FlutterMethodChannel methodChannelWithName:viewIdStr binaryMessenger:registrar.messenger];
    WeakSelf
    [_channel setMethodCallHandler:^(FlutterMethodCall * _Nonnull call, FlutterResult  _Nonnull result) {
        if ([call.method isEqualToString:@"toggleTorchMode"]) {//开关
            [weakSelf turnTorchOn];
        }else if ([call.method isEqualToString:@"beginScan"]){//开启扫码
            [weakSelf setupCamera];
        }else if ([call.method isEqualToString:@"resetStart"]){//重新开启扫码器
            [weakSelf.session startRunning];
        }else if ([call.method isEqualToString:@"disposeScan"]){//清除扫码器
            [weakSelf killScan];
        }
    }];
}

-(void)turnTorchOn
{
    _isFlashLight = !_isFlashLight;
    Class captureDeviceClass = NSClassFromString(@"AVCaptureDevice");
    if (captureDeviceClass !=nil) {
        AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
        if ([device hasTorch] && [device hasFlash]){
            [device lockForConfiguration:nil];
            if (_isFlashLight) {
                [device setTorchMode:AVCaptureTorchModeOn];
                [device setFlashMode:AVCaptureFlashModeOn];
            } else {
                [device setTorchMode:AVCaptureTorchModeOff];
                [device setFlashMode:AVCaptureFlashModeOff];
            }
            [device unlockForConfiguration];
        }else{
            NSLog(@"初始化失败");
        }
    }else{
        NSLog(@"没有闪光设备");
    }
}

- (void)setupCamera {
    WeakSelf
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        
        weakSelf.device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
        
        weakSelf.input = [AVCaptureDeviceInput deviceInputWithDevice:weakSelf.device error:nil];
        
        weakSelf.output = [[AVCaptureMetadataOutput alloc]init];
        [weakSelf.output setMetadataObjectsDelegate:weakSelf queue:dispatch_get_main_queue()];
        
        weakSelf.session = [[AVCaptureSession alloc]init];
        [weakSelf.session setSessionPreset:AVCaptureSessionPresetHigh];
        if ([weakSelf.session canAddInput:weakSelf.input])
        {
            [weakSelf.session addInput:weakSelf.input];
        }
        
        if ([weakSelf.session canAddOutput:weakSelf.output])
        {
            [weakSelf.session addOutput:weakSelf.output];
        }
        
        // 条码类型 AVMetadataObjectTypeQRCode
        weakSelf.output.metadataObjectTypes = [NSArray arrayWithObjects:AVMetadataObjectTypeEAN13Code, AVMetadataObjectTypeEAN8Code, AVMetadataObjectTypeCode128Code, AVMetadataObjectTypeQRCode, nil];
        [weakSelf.output setRectOfInterest:CGRectMake(0,0,SCREEN_WIDTH,SCREENH_HEIGHT)];
        dispatch_async(dispatch_get_main_queue(), ^{
            //更新界面
            weakSelf.preview =[AVCaptureVideoPreviewLayer layerWithSession:weakSelf.session];
            weakSelf.preview.videoGravity = AVLayerVideoGravityResizeAspectFill;
            weakSelf.preview.frame = CGRectMake(0, 0, SCREEN_WIDTH, SCREENH_HEIGHT);
            [weakSelf.view.layer insertSublayer:weakSelf.preview atIndex:0];
            [weakSelf.session startRunning];
        });
    });
}

-(void)stopReading
{
    [_session stopRunning];
//    _session = nil;
//    [_preview removeFromSuperlayer];
    
}

-(void)killScan{
    [self stopReading];
    _session = nil;
    [_preview removeFromSuperlayer];
}

-(void)captureOutput:(AVCaptureOutput *)output didOutputMetadataObjects:(NSArray<__kindof AVMetadataObject *> *)metadataObjects fromConnection:(AVCaptureConnection *)connection{
    NSString *stringValue;
    if ([metadataObjects count] >0){
        AVMetadataMachineReadableCodeObject * metadataObject = [metadataObjects objectAtIndex:0];
        stringValue = metadataObject.stringValue;
        NSLog(@"%@",stringValue);
        
        [self.channel invokeMethod:@"onCaptured" arguments:stringValue];
    }
    
    [self stopReading];
}



-(UIView *)view{
    return _bigView;
}

@end
